"""Render an animation."""

from indiepixel import Animation, Rect


def main():
    """
    Render the widget.

    This matches the example in https://github.com/tidbyt/pixlet/blob/main/docs/widgets.md#stack
    Successfully matches!
    """

    return Animation(
        debug_label="Outer",
        children=[
            Rect(width=10, height=10, color="#f00"),
            Rect(width=12, height=12, color="#0ff"),
            Rect(width=14, height=14, color="#00f"),
            Animation(
                children=[
                    Rect(width=12, height=12, color="#00f"),
                    Rect(width=8, height=8, color="#00f"),
                    Rect(width=4, height=4, color="#00f"),
                    Rect(width=2, height=2, color="#00f"),
                ]
            ),
            Rect(width=16, height=16, color="#0f0"),
            Rect(width=18, height=18, color="#ff0"),
            Animation(
                children=[
                    Rect(width=12, height=12, color="#ff0"),
                    Rect(width=8, height=8, color="#ff0"),
                    Rect(width=4, height=4, color="#ff0"),
                    Rect(width=2, height=2, color="#ff0"),
                ]
            ),
        ],
    )



"""CLI that runs an interactive server for indiepixel."""

import os
import sys
from importlib.util import module_from_spec, spec_from_file_location
from io import BytesIO
from pathlib import Path

import click
from flask import Flask, render_template, send_file

from indiepixel import render


def create_server(filename: str, duration: int):
    """Produce a server that will display the widgets listed by mods."""
    app = Flask(__name__)

    @app.route("/")
    def root() -> str:
        mods = load_from_path(filename)
        return render_template("index.html", widgets=mods)

    @app.route("/widget/<path:subpath>")
    def widget(subpath) -> str:
        mods = load_from_path(filename)
        widget = next((w for w in mods if w[0] == subpath), None)
        if widget is None:
            return render_template("not_found.html")
        return render_template("widget.html", widget=widget)

    @app.route("/image/<path:subpath>.webp")
    def image(subpath):
        mods = load_from_path(filename)
        """Display an image."""
        mod = [x for x in mods if x[0] == subpath]
        if len(mod) == 0:
            return f"Image not found, available modules are {map(lambda m: m[0], mods)}"
        frames = render(mod[0][1].main())
        # It's very important to specify lossless=True here,
        # otherwise we get blurry output
        img_io = BytesIO()
        print(frames)
        frames[0].save(
            img_io,
            "WEBP",
            lossless=True,
            alpha_quality=100,
            save_all=True,
            append_images=frames[1:],
            duration=duration,
            loop=0,
        )
        img_io.seek(0)
        return send_file(img_io, mimetype="image/webp")

    return app


def import_from_path(module_name, file_path):
    """Import a module given its name and file path."""
    print(f"importing {file_path}")
    spec = spec_from_file_location(module_name, file_path)
    if spec is None or spec.loader is None:
        raise Exception("Could not get spec")
    module = module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    return module


def load_from_path(filename):
    """Find all python files in a path if it's a directory."""
    if os.path.isdir(filename):
        files = Path(filename).glob("*.py")
        return [(str(p), import_from_path("render", p)) for p in files]
    return [(filename, import_from_path("render", filename))]


@click.command()
@click.argument("filename")
@click.option("--duration", default=500, help="Frame duration for animations")
def cli(filename: str, duration: int):
    """Run indiepixel in a CLI."""
    app = create_server(filename, duration)
    print("created app", app)
    app.run(debug=True)



"""Render an animation."""

from indiepixel import Animation, Rect


def main():
    """
    Render the widget.

    This matches the example in https://github.com/tidbyt/pixlet/blob/main/docs/widgets.md#stack
    Successfully matches!
    """
    return Animation(
        children=[
            Rect(width=10, height=10, color="#f00"),
            Rect(width=12, height=12, color="#0ff"),
            Rect(width=14, height=14, color="#00f"),
            Rect(width=16, height=16, color="#0f0"),
            Rect(width=18, height=18, color="#ff0"),
        ],
    )



"""Render a gradient."""

from indiepixel import Box, Column, Rect, Row


def main():
    """Render the widget."""
    return Box(
        Column(
            children=[
                Row(
                    children=[
                        Rect(height=16, width=1, color=(0, 0, i * 4)) for i in range(64)
                    ]
                ),
                Row(
                    children=[
                        Rect(height=16, width=1, color=(255 - (i * 4), 0, i * 4))
                        for i in range(64)
                    ]
                ),
            ]
        ),
        background="#000",
    )



"""Render a clock."""

import datetime

import pytz

# import pytz
from indiepixel import Box, Row, Text


def main():
    """Render a clock."""
    tz = pytz.timezone("America/New_York")
    now = datetime.datetime.now(tz)
    return Box(
        Row(
            children=[
                Text(content=now.strftime("%I:%M"), color="#fff"),
            ]
        ),
        padding=2,
        background="#000",
    )



"""Render a gradient."""

from indiepixel import Box, Circle


def main():
    """
    Render the widget.

    This matches the example in https://github.com/tidbyt/pixlet/blob/main/docs/widgets.md#circle
    Note that it's not exactly the same rendering
    because pixlet does anti-aliasing and this does
    not, currently.
    """
    return Box(
        Circle(
            color="#666",
            diameter=30,
            child=Circle(color="#0ff", diameter=10),
        ),
        background="#000",
    )



"""Render a gradient."""

from indiepixel import PieChart


def main():
    """
    Render the widget.

    https://github.com/tidbyt/pixlet/blob/main/docs/widgets.md#piechart
    """
    return PieChart(
        colors=["#fff", "#0f0", "#00f"],
        weights=[180, 135, 45],
        diameter=30,
    )



"""Bdf font parsing."""
#
# ruff: noqa: D103
#
# The Python Imaging Library
# $Id$
#
# PIL raster font compiler
#
# history:
# 1997-08-25 fl   created
# 2002-03-10 fl   use "from PIL import"
#

import glob
import sys

# drivers
from PIL import BdfFontFile, PcfFontFile

VERSION = "0.4"

if len(sys.argv) <= 1:
    sys.exit(1)

files = []
for f in sys.argv[1:]:
    files = files + glob.glob(f)

for f in files:
    try:
        with open(f, "rb") as fp:
            try:
                p = PcfFontFile.PcfFontFile(fp)
            except SyntaxError:
                fp.seek(0)
                p = BdfFontFile.BdfFontFile(fp)

            p.save(f)

    except (OSError, SyntaxError):
        pass

    else:
        pass



"""Render a gradient."""

from indiepixel import WrappedText


def main():
    """
    Render the widget.

    This matches the example in https://github.com/tidbyt/pixlet/blob/main/docs/widgets.md#stack
    Successfully matches!
    """
    return WrappedText(
        content="this is a multi-line text string",
        color="#fa0",
        align="center",
    )



"""Tests for widgets."""

# ruff: noqa: D103
from __future__ import annotations

from io import BytesIO

import pytest
from PIL import Image as ImagePIL
from PIL import ImageDraw
from syrupy.extensions.image import PNGImageSnapshotExtension

from indiepixel import Box, Renderable, Row, Text


# Make syrupy save snapshots as individual png files
@pytest.fixture
def snapshot(snapshot):
    return snapshot.use_extension(PNGImageSnapshotExtension)


@pytest.fixture
def image() -> ImagePIL.Image:
    return ImagePIL.new("RGB", (32, 16))


@pytest.fixture
def image_draw(image: ImagePIL.Image) -> ImageDraw.ImageDraw:
    return ImageDraw.ImageDraw(image)


@pytest.fixture
def render_widget(image, image_draw):
    """Render a widget to PNG image data."""

    def render_widget_impl(widget: Renderable) -> bytes:
        widget.paint(image_draw, image, (0, 0, 64, 32), 0)

        buffer = BytesIO()
        image.save(buffer, format="PNG")
        png_bytes = buffer.getvalue()
        buffer.close()
        return png_bytes

    return render_widget_impl


def test_clock(snapshot, render_widget) -> None:
    widget = Box(
        Row(
            children=[
                Text(content="1:23AM", color="#fff"),
            ]
        ),
        padding=2,
        background="#000",
    )
    assert render_widget(widget) == snapshot



"""Render an animation."""

from indiepixel import Animation, Image
from pathlib import Path


def main():
    """
    Render the image.

    Thanks to https://itch.io/queue/c/4949099/16x16-game-assets?game_id=1441781
    asset is CC0. Buy their art!
    """
    return Image(src=Path(__file__).resolve().parent / "tree.png")



"""Render a gradient."""

from indiepixel import Rect, Stack, Text


def main():
    """
    Render the widget.

    This matches the example in https://github.com/tidbyt/pixlet/blob/main/docs/widgets.md#stack
    Successfully matches!
    """
    return Stack(
        children=[
            Rect(width=50, height=25, color="#911"),
            Text(content="hello there"),
            Rect(width=4, height=32, color="#119"),
        ],
    )
